@@ -35,6 +35,7 @@
 #include "php_igbinary.h"
 
 #include "igbinary.h"
+#include "igbinary_macros.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -438,7 +439,7 @@ static int igbinary_finish_wakeup(struct igbinary_unserialize_data* igsd) {
 		zend_object *obj = igsd->wakeup[i];
 		ZVAL_OBJ(&rval, obj);
 		if (call_user_function_ex(CG(function_table), &rval, &fname, &retval, 0, 0, 1, NULL) == FAILURE || Z_ISUNDEF(retval)) {
-			GC_FLAGS(obj) |= IS_OBJ_DESTRUCTOR_CALLED;
+			GC_ADD_FLAGS(obj, IS_OBJ_DESTRUCTOR_CALLED);
 		}
 		zval_ptr_dtor(&retval);
 		if (EG(exception)) {
@@ -446,7 +447,7 @@ static int igbinary_finish_wakeup(struct igbinary_unserialize_data* igsd) {
 			zval_dtor(&fname);
 			/* Don't call __destruct for any of the objects which __wakeup wasn't called on yet, either */
 			for (j = i + 1; j < igsd->wakeup_count; j++) {
-				GC_FLAGS(igsd->wakeup[j]) |= IS_OBJ_DESTRUCTOR_CALLED;
+				GC_ADD_FLAGS(igsd->wakeup[j], IS_OBJ_DESTRUCTOR_CALLED);
 			}
 			return 1;
 		}
@@ -1334,6 +1335,11 @@ inline static int igbinary_serialize_array_sleep(struct igbinary_serialize_data
 				if (Z_TYPE_P(v) == IS_INDIRECT) {
 					v = Z_INDIRECT_P(v);
 				}
+				if (UNEXPECTED(Z_TYPE_P(v) == IS_UNDEF)) {
+					php_error_docref(NULL, E_NOTICE, "\"%s\" returned as member variable from __sleep() but does not exist", Z_STRVAL_P(d));
+					igbinary_serialize_null(igsd);
+					continue;
+				}
 				if (igbinary_serialize_zval(igsd, v) != 0) {
 					return 1;
 				}
@@ -1384,6 +1390,12 @@ inline static int igbinary_serialize_array_sleep(struct igbinary_serialize_data
 					}
 
 					zend_string_release(mangled_prop_name);
+
+					if (UNEXPECTED(Z_TYPE_P(v) == IS_UNDEF)) {
+						php_error_docref(NULL, E_NOTICE, "\"%s\" returned as member variable from __sleep() but does not exist", Z_STRVAL_P(d));
+						igbinary_serialize_null(igsd);
+						break;
+					}
 					if (igbinary_serialize_zval(igsd, v) != 0) {
 						return 1;
 					}
@@ -1676,8 +1688,8 @@ static int igbinary_serialize_zval(struct igbinary_serialize_data *igsd, zval *z
 		case IS_NULL:
 			return igbinary_serialize_null(igsd);
 		case IS_UNDEF:
-			// As of php 7.1.3, started seeing "zval has unknown type 0"
-			zend_error(E_WARNING, "igbinary_serialize_zval: zval has unexpected type IS_UNDEF(0)");
+			// https://github.com/igbinary/igbinary/issues/134
+			// TODO: In a new major version, could have a separate type for IS_UNDEF, which would unset the property in an object context?
 			return igbinary_serialize_null(igsd);
 		case IS_TRUE:
 			return igbinary_serialize_bool(igsd, 1);
@@ -1697,7 +1709,7 @@ static int igbinary_serialize_zval(struct igbinary_serialize_data *igsd, zval *z
 /* {{{ igbinary_unserialize_data_init */
 /** Inits igbinary_unserialize_data_init. */
 inline static int igbinary_unserialize_data_init(struct igbinary_unserialize_data *igsd) {
-	smart_string empty_str = { 0 };
+	smart_string empty_str = {0, 0, 0};
 
 	igsd->buffer = NULL;
 	igsd->buffer_end = NULL;
@@ -1927,7 +1939,7 @@ inline static int igbinary_unserialize_long(struct igbinary_unserialize_data *ig
 #endif
 	} else {
 		*ret = 0;
-		zend_error(E_WARNING, "igbinary_unserialize_long: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_long: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return 1;
 	}
 
@@ -1981,7 +1993,7 @@ inline static zend_string* igbinary_unserialize_string(struct igbinary_unseriali
 		}
 		i = igbinary_unserialize32(igsd);
 	} else {
-		zend_error(E_WARNING, "igbinary_unserialize_string: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_string: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return NULL;
 	}
 
@@ -2042,7 +2054,7 @@ inline static zend_string* igbinary_unserialize_chararray(struct igbinary_unseri
 			return NULL;
 		}
 	} else {
-		zend_error(E_WARNING, "igbinary_unserialize_chararray: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_chararray: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return NULL;
 	}
 
@@ -2106,13 +2118,13 @@ inline static int igbinary_unserialize_array(struct igbinary_unserialize_data *i
 		}
 		n = igbinary_unserialize32(igsd);
 	} else {
-		zend_error(E_WARNING, "igbinary_unserialize_array: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_array: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return 1;
 	}
 
 	/* n cannot be larger than the number of minimum "objects" in the array */
 	if (IGB_NEEDS_MORE_DATA(igsd, n)) {
-		zend_error(E_WARNING, "igbinary_unserialize_array: data size %zu smaller that requested array length %zu.", IGB_REMAINING_BYTES(igsd), (unsigned int) n);
+		zend_error(E_WARNING, "igbinary_unserialize_array: data size %zu smaller that requested array length %zu.", (size_t)IGB_REMAINING_BYTES(igsd), (size_t) n);
 		return 1;
 	}
 
@@ -2200,7 +2212,7 @@ inline static int igbinary_unserialize_array(struct igbinary_unserialize_data *i
 			case igbinary_type_null:
 				continue;
 			default:
-				zend_error(E_WARNING, "igbinary_unserialize_array: unknown key type '%02x', position %zu", key_type, IGB_BUFFER_OFFSET(igsd));
+				zend_error(E_WARNING, "igbinary_unserialize_array: unknown key type '%02x', position %zu", key_type, (size_t)IGB_BUFFER_OFFSET(igsd));
 				zval_dtor(z);
 				ZVAL_UNDEF(z);
 				return 1;
@@ -2269,13 +2281,13 @@ inline static int igbinary_unserialize_object_properties(struct igbinary_unseria
 		}
 		n = igbinary_unserialize32(igsd);
 	} else {
-		zend_error(E_WARNING, "igbinary_unserialize_array: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_array: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return 1;
 	}
 
 	/* n cannot be larger than the number of minimum "objects" in the array */
 	if (IGB_NEEDS_MORE_DATA(igsd, n)) {
-		zend_error(E_WARNING, "%s: data size %zu smaller that requested array length %zu.", "igbinary_unserialize_array", IGB_REMAINING_BYTES(igsd), n);
+		zend_error(E_WARNING, "%s: data size %zu smaller that requested array length %zu.", "igbinary_unserialize_array", (size_t)IGB_REMAINING_BYTES(igsd), (size_t)n);
 		return 1;
 	}
 
@@ -2353,7 +2365,7 @@ inline static int igbinary_unserialize_object_properties(struct igbinary_unseria
 			case igbinary_type_null:
 				continue;  /* Skip unserializing this element, serialized with no value. In C, this applies to loop, not switch. */
 			default:
-				zend_error(E_WARNING, "igbinary_unserialize_array: unknown key type '%02x', position %zu", key_type, IGB_BUFFER_OFFSET(igsd));
+				zend_error(E_WARNING, "igbinary_unserialize_array: unknown key type '%02x', position %zu", key_type, (size_t)IGB_BUFFER_OFFSET(igsd));
 				zval_dtor(z);
 				ZVAL_UNDEF(z);
 				return 1;
@@ -2407,7 +2419,8 @@ inline static int igbinary_unserialize_object_ser(struct igbinary_unserialize_da
 	php_unserialize_data_t var_hash;
 
 	if (ce->unserialize == NULL) {
-		zend_error(E_WARNING, "Class %s has no unserializer", ce->name);
+		// Should be impossible.
+		zend_error(E_WARNING, "Class %s has no unserializer", ZSTR_VAL(ce->name));
 		return 1;
 	}
 
@@ -2430,7 +2443,7 @@ inline static int igbinary_unserialize_object_ser(struct igbinary_unserialize_da
 		}
 		n = igbinary_unserialize32(igsd);
 	} else {
-		zend_error(E_WARNING, "igbinary_unserialize_object_ser: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_object_ser: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return 1;
 	}
 
@@ -2479,7 +2492,7 @@ inline static int igbinary_unserialize_object(struct igbinary_unserialize_data *
 	} else if (t == igbinary_type_object_id8 || t == igbinary_type_object_id16 || t == igbinary_type_object_id32) {
 		class_name = igbinary_unserialize_string(igsd, t);
 	} else {
-		zend_error(E_WARNING, "igbinary_unserialize_object: unknown object type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_object: unknown object type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return 1;
 	}
 
@@ -2545,7 +2558,8 @@ inline static int igbinary_unserialize_object(struct igbinary_unserialize_data *
 	}
 
 	{
-		struct igbinary_value_ref ref;
+		// The actual value of ref is unused. We use ref_n later in this function, after creating the object.
+		struct igbinary_value_ref ref = {{0}, 0};
 		ref_n = igsd_append_ref(igsd, ref);
 		if (ref_n == SIZE_MAX) {
 			zend_string_release(class_name);
@@ -2606,7 +2620,7 @@ inline static int igbinary_unserialize_object(struct igbinary_unserialize_data *
 			break;
 		}
 		default:
-			zend_error(E_WARNING, "igbinary_unserialize_object: unknown object inner type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+			zend_error(E_WARNING, "igbinary_unserialize_object: unknown object inner type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 			r = 1;
 	}
 	zend_string_release(class_name);
@@ -2624,12 +2638,12 @@ inline static int igbinary_unserialize_object(struct igbinary_unserialize_data *
 		} else if (ref->type == IG_REF_IS_REFERENCE) {
 			ztemp = ref->reference.reference->val;
 			if (Z_TYPE(ztemp) != IS_OBJECT) {
-				zend_error(E_WARNING, "igbinary_unserialize_object preparing to __wakeup: got reference to non-object somehow", t, IGB_BUFFER_OFFSET(igsd));
+				zend_error(E_WARNING, "igbinary_unserialize_object preparing to __wakeup: got reference to non-object somehow (inner type '%02x', position %zu)", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 				return 1;
 			}
 			object = Z_OBJ(ztemp);
 		} else {
-			zend_error(E_WARNING, "igbinary_unserialize_object preparing to __wakeup: created non-object somehow", t, IGB_BUFFER_OFFSET(igsd));
+			zend_error(E_WARNING, "igbinary_unserialize_object preparing to __wakeup: created non-object somehow (inner type '%02x', position %zu)", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 			return 1;
 		}
 		zend_class_entry *ztemp_ce;
@@ -2672,12 +2686,12 @@ inline static int igbinary_unserialize_ref(struct igbinary_unserialize_data *igs
 		}
 		n = igbinary_unserialize32(igsd);
 	} else {
-		zend_error(E_WARNING, "igbinary_unserialize_ref: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+		zend_error(E_WARNING, "igbinary_unserialize_ref: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 		return 1;
 	}
 
 	if (n >= igsd->references_count) {
-		zend_error(E_WARNING, "igbinary_unserialize_ref: invalid reference %zu >= %zu", (int) n, (int)igsd->references_count);
+		zend_error(E_WARNING, "igbinary_unserialize_ref: invalid reference %zu >= %zu", (size_t) n, (size_t)igsd->references_count);
 		return 1;
 	}
 
@@ -2870,7 +2884,7 @@ static int igbinary_unserialize_zval(struct igbinary_unserialize_data *igsd, zva
 			ZVAL_DOUBLE(z, tmp_double);
 			break;
 		default:
-			zend_error(E_WARNING, "igbinary_unserialize_zval: unknown type '%02x', position %zu", t, IGB_BUFFER_OFFSET(igsd));
+			zend_error(E_WARNING, "igbinary_unserialize_zval: unknown type '%02x', position %zu", t, (size_t)IGB_BUFFER_OFFSET(igsd));
 			return 1;
 	}
 
